﻿// ***********************************************************************************
// SallySoundFX
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// https://www.patreon.com/sallyw
// Licensed under CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/

using System;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;

namespace Sally
{
    public class AVSLAVE_Bands : MVRScript
    {
		// UI
		private JSONStorableString jsBenchmark;
		private JSONStorableColor jColorHi;
		private JSONStorableColor jColorLo;
		private JSONStorableColor jColorPeaks;

		private JSONStorableBool jbCollision;
		private JSONStorableBool jbPeaks;
		private JSONStorableFloat jfCubeScale;
		private JSONStorableFloat jfCubeXOffset;
		private JSONStorableFloat jfHeightScale;

		// BENCHMARK keep namespace, can cause VAM compilation error
		private System.Diagnostics.Stopwatch stopWatch = new System.Diagnostics.Stopwatch();
		private double[] avgUpdateTimes = new double[80];
		private double avgUpdateTime = 0;

		// AUDIO VISUALIZATION DATA
		private AVD avd = new AVD();

		// VISUALIZATION CUBES
		private GameObject[] cubes = new GameObject[AVD.BANDS];
		private MeshRenderer[] cubesMeshRenderers = new MeshRenderer[AVD.BANDS];
		private MeshFilter[] cubesMeshFilters = new MeshFilter[AVD.BANDS];
		private Material cubeMaterial;
		// VISUALIZATION CUBES PEAK
		private GameObject[] cubesPeak = new GameObject[AVD.BANDS];
		private MeshRenderer[] cubesPeakMeshRenderers = new MeshRenderer[AVD.BANDS];
		private MeshFilter[] cubesPeakMeshFilters = new MeshFilter[AVD.BANDS];
		private Material cubePeakMaterial;

		// MISCELLANEOUS
		float v; // optimization
		Vector3 newScale; // used to scale bands to their value
		Vector3 newPosition; // used to position peaks on top of bands
		Color newColor = Color.white;
		Color[] colors = new Color[24]; // colors.Length must be mesh.vertices.Length!
		Color colorLoRGB = Color.cyan;
		Color colorHiRGB = Color.magenta;
		Color colorPeaksRGB = Color.white;
		public override void Init()
		{
			try
			{
				if (AVSlave.GetMasterScript(this) == null)
					SuperController.LogMessage($"{name} could not find SallySoundFX Plugin to synchronize. It will retry automatically.");

				//////////////////////////////////////////////////////////////////////////////////
				// UI - LEFT
				//////////////////////////////////////////////////////////////////////////////////
				//Vector3 selectorPosition;

				// Color Picker High
				jColorHi = new JSONStorableColor("Color High", HSVColorPicker.RGBToHSV(1f, 0f, 1f), SetColorHi);
				CreateColorPicker(jColorHi);
				RegisterColor(jColorHi);
				SetColorHi(jColorHi.val.H, jColorHi.val.S, jColorHi.val.V);
				UI.FixInitialColorPickerSelectorPosition(jColorHi);

				// Color Picker Low
				jColorLo = new JSONStorableColor("Color Low", HSVColorPicker.RGBToHSV(0f, 1f, 1f), SetColorLo);
				CreateColorPicker(jColorLo);
				RegisterColor(jColorLo);
				SetColorLo(jColorLo.val.H, jColorLo.val.S, jColorLo.val.V);
				UI.FixInitialColorPickerSelectorPosition(jColorLo);

				// Color Picker Peaks
				jColorPeaks = new JSONStorableColor("Color Peaks", HSVColorPicker.RGBToHSV(1f, 1f, 1f), SetColorPeaks);
				CreateColorPicker(jColorPeaks);
				RegisterColor(jColorPeaks);
				colorPeaksRGB = Color.HSVToRGB(jColorPeaks.val.H, jColorPeaks.val.S, jColorPeaks.val.V);
				UI.FixInitialColorPickerSelectorPosition(jColorPeaks);

				//////////////////////////////////////////////////////////////////////////////////
				// RIGHT UI
				//////////////////////////////////////////////////////////////////////////////////

				// Benchmark
				jsBenchmark = new JSONStorableString("Benchmark", "no data");
				UIDynamicTextField tf = CreateTextField(jsBenchmark, true);
				LayoutElement le = tf.GetComponent<LayoutElement>();
				le.minHeight = 40.0f;
				le.preferredHeight = 40.0f;

				// Collision
				jbCollision = new JSONStorableBool("Collision (Peaks excluded)", false);
				RegisterBool(jbCollision);
				UIDynamicToggle toggle = CreateToggle(jbCollision, true);
				toggle.backgroundColor = UI.redWarningBackground;
				jbCollision.setCallbackFunction += RebuildSpectrumBool;

				// Peaks
				jbPeaks = new JSONStorableBool("Peaks", true);
				RegisterBool(jbPeaks);
				CreateToggle(jbPeaks, true);
				jbPeaks.setCallbackFunction += SetPeaksActive;

				// Slider Cube Scale
				jfCubeScale = new JSONStorableFloat("Cube Scale", 0.1f, 0.001f, 1.0f);
				RegisterFloat(jfCubeScale);
				UIDynamicSlider sliderCubeScale = CreateSlider(jfCubeScale, true);
				sliderCubeScale.Configure(jfCubeScale.name, jfCubeScale.min, jfCubeScale.max, jfCubeScale.val, jfCubeScale.constrained, "F4", true, false);
				// safe to assume to return the "Panel" image first?
				sliderCubeScale.GetComponentInChildren<Image>().color = UI.redWarningBackground;
				//var images = sliderCubeScale.GetComponentsInChildren<Image>();
				//Image panel = images.First(image => image.name == "Panel");
				//panel.color = UI.redWarningBackground;
				jfCubeScale.setCallbackFunction += RebuildSpectrum;

				// Slider Cube X Offset
				jfCubeXOffset = new JSONStorableFloat("Cube X Offset", 0.01f, 0.0f, 0.5f);
				RegisterFloat(jfCubeXOffset);
				UIDynamicSlider sliderCubeXOffset = CreateSlider(jfCubeXOffset, true);
				sliderCubeXOffset.Configure(jfCubeXOffset.name, jfCubeXOffset.min, jfCubeXOffset.max, jfCubeXOffset.val, jfCubeXOffset.constrained, "F4", true, false);
				sliderCubeXOffset.GetComponentInChildren<Image>().color = UI.redWarningBackground;
				jfCubeXOffset.setCallbackFunction += RebuildSpectrum;

				// Slider Height Scale
				jfHeightScale = new JSONStorableFloat("Height Scale", 1.0f, 0.1f, 5.0f);
				RegisterFloat(jfHeightScale);
				CreateSlider(jfHeightScale, true);

				//////////////////////////////////////////////////////////////////////////////////
				// VISUALIZATION
				//////////////////////////////////////////////////////////////////////////////////

				CreateVisualization(true);
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		private void CreateVisualization(bool initialize = false)
		{
			// Atom rotation workaround - save original state
			// makes the Visualization stay in sync with Atom-rotation after scene load
			Quaternion rotationOriginal = containingAtom.mainController.transform.rotation;
			Quaternion localRotationOriginal = containingAtom.mainController.transform.localRotation;
			// reset rotation
			containingAtom.mainController.transform.rotation = Quaternion.identity;
			containingAtom.mainController.transform.localRotation = Quaternion.identity;

			// CUBES
			newScale = new Vector3(jfCubeScale.val, jfCubeScale.val, jfCubeScale.val);
			newPosition = new Vector3(0f, 0f, 0f);
			if (initialize)
			{
				cubeMaterial = new Material(Shader.Find("Sprites/Default")); // Sprites/Default  -  Custom/Subsurface/EmissiveGlow
				cubePeakMaterial = new Material(Shader.Find("Sprites/Default"));
			}
			float x = 0.0f;
			float xoffset = jfCubeScale.val + jfCubeXOffset.val;
			Vector3 initialScale = new Vector3(jfCubeScale.val, jfCubeScale.val, jfCubeScale.val);
			for (int i = 0; i < cubes.Length; i++)
			{
				// Bands
				if (initialize)
					cubes[i] = GameObject.CreatePrimitive(PrimitiveType.Cube);
				cubes[i].transform.parent = containingAtom.mainController.transform;
				cubes[i].transform.position = containingAtom.mainController.transform.position + new Vector3(-x, 0.0f, 0.0f);
				cubes[i].transform.localScale = initialScale;
				//cubes[i].transform.name = "cube" + i;
				cubesMeshRenderers[i] = cubes[i].GetComponent<MeshRenderer>();
				cubesMeshRenderers[i].material = cubeMaterial;
				cubesMeshFilters[i] = cubes[i].GetComponent<MeshFilter>();
				// remove Box Collider
				if (!jbCollision.val)
					UnityEngine.Object.DestroyImmediate(cubes[i].GetComponent<Collider>());

				// Peaks
				if (jbPeaks.val)
				{
					if (initialize)
						cubesPeak[i] = GameObject.CreatePrimitive(PrimitiveType.Cube);
					cubesPeak[i].transform.parent = containingAtom.mainController.transform;
					cubesPeak[i].transform.position = containingAtom.mainController.transform.position + new Vector3(-x, 0.0f, 0.0f);
					cubesPeak[i].transform.localScale = initialScale;
					cubesPeakMeshRenderers[i] = cubesPeak[i].GetComponent<MeshRenderer>();
					cubesPeakMeshRenderers[i].material = cubePeakMaterial;
					cubesPeakMeshFilters[i] = cubesPeak[i].GetComponent<MeshFilter>();

					// set vertex colors
					for (int c = 0; c < colors.Length; c++)
						colors[c] = colorPeaksRGB;
					cubesPeakMeshFilters[i].mesh.colors = colors; // slow

					// don't have Colliders on Peaks
					UnityEngine.Object.DestroyImmediate(cubesPeak[i].GetComponent<Collider>());
				}

				x += xoffset;
			}

			// Atom rotation workaround - restore original state
			containingAtom.mainController.transform.rotation = rotationOriginal;
			containingAtom.mainController.transform.localRotation = localRotationOriginal;
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start()
		{
			try
			{
				// put code in here
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		// Update is called with each rendered frame by Unity
		void Update()
		{
			try
			{
				// BENCHMARK START
				stopWatch.Reset();
				stopWatch.Start();

				if (!AVSlave.IsSynchronized(this))
					return;

				int c; // optimize
				float v_inverted; // optimize
				float cubeScaleHalf = jfCubeScale.val * 0.5f; // optimize
				for (int i = 0; i < avd.bands.Length; i++)
				{
					v = avd.bands[i];
					v_inverted = 1f - v;
					newScale.y = jfCubeScale.val + jfHeightScale.val * v;
					newColor.r = colorLoRGB.r * v_inverted + colorHiRGB.r * v;
					newColor.g = colorLoRGB.g * v_inverted + colorHiRGB.g * v;
					newColor.b = colorLoRGB.b * v_inverted + colorHiRGB.b * v;
					cubesMeshRenderers[i].transform.localScale = newScale;
					// set material color
					//meshRenderers[i].material.color = newColor; // 0.120 ms per loop

					// set vertex colors
					for (c = 0; c < colors.Length; c++)
						colors[c] = newColor;
					cubesMeshFilters[i].mesh.colors = colors; // slow

					// Peaks
					if (jbPeaks.val)
					{
						v = avd.bands_peak[i];
						newPosition = cubesMeshRenderers[i].transform.localPosition; 
						newPosition.y += (jfCubeScale.val + jfHeightScale.val * v) * 0.5f + cubeScaleHalf;
						cubesPeakMeshRenderers[i].transform.localPosition = newPosition;

						// set vertex colors
						for (c = 0; c < colors.Length; c++)
							colors[c] = colorPeaksRGB;
						cubesPeakMeshFilters[i].mesh.colors = colors; // slow
					}
				}

				// BENCHMARK END
				stopWatch.Stop();
				avgUpdateTimes[avgUpdateTimes.Length - 1] = stopWatch.Elapsed.TotalMilliseconds;
				Array.Copy(avgUpdateTimes, 1, avgUpdateTimes, 0, avgUpdateTimes.Length - 1); // shift array
				avgUpdateTime = 0;
				for (int i = 0; i < avgUpdateTimes.Length; i++)
					avgUpdateTime += avgUpdateTimes[i];
				avgUpdateTime = avgUpdateTime / avgUpdateTimes.Length;

				jsBenchmark.val = $"Update Benchmark: {avgUpdateTime:F4} ms";

			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate()
		{
			try
			{
				// put code in here
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy()
		{
			int i;
			// cubes
			for (i = 0; i < cubes.Length; i++)
			{
				Destroy(cubes[i]);
				cubes[i] = null;
			}
			cubes = null;

			// cubesMeshRenderers
			for (i = 0; i < cubesMeshRenderers.Length; i++)
			{
				Destroy(cubesMeshRenderers[i]);
				cubesMeshRenderers[i] = null;
			}
			cubesMeshRenderers = null;

			// cubesMeshFilters
			for (i = 0; i < cubesMeshFilters.Length; i++)
			{
				Destroy(cubesMeshFilters[i]);
				cubesMeshFilters[i] = null;
			}
			cubesMeshFilters = null;

			// cubeMaterial
			if (cubeMaterial != null)
			{
				Destroy(cubeMaterial);
				cubeMaterial = null;
			}

			// cubesPeak
			for (i = 0; i < cubesPeak.Length; i++)
			{
				Destroy(cubesPeak[i]);
				cubesPeak[i] = null;
			}
			cubesPeak = null;

			// cubesPeakMeshRenderers
			for (i = 0; i < cubesPeakMeshRenderers.Length; i++)
			{
				Destroy(cubesPeakMeshRenderers[i]);
				cubesPeakMeshRenderers[i] = null;
			}
			cubesPeakMeshRenderers = null;

			// cubesPeakMeshFilters
			for (i = 0; i < cubesPeakMeshFilters.Length; i++)
			{
				Destroy(cubesPeakMeshFilters[i]);
				cubesPeakMeshFilters[i] = null;
			}
			cubesPeakMeshFilters = null;

			// cubePeakMaterial
			if (cubePeakMaterial != null)
			{
				Destroy(cubePeakMaterial);
				cubePeakMaterial = null;
			}

			avgUpdateTimes = null;
			avd = null;
		}

		//////////////////////////////////////////////////////////////////////////////////
		// UI Callbacks
		//////////////////////////////////////////////////////////////////////////////////
		private void RebuildSpectrumBool(bool b) { CreateVisualization(); }
		private void RebuildSpectrum(float s) { CreateVisualization(); }
		private void SetPeaksActive(bool b)
        {
			for (int i = 0; i < cubesPeak.Length; i++)
				cubesPeak[i].SetActive(b);
		}
		private void SetColorHi(float h, float s, float v) { colorHiRGB = Color.HSVToRGB(h, s, v); }
		private void SetColorLo(float h, float s, float v) { colorLoRGB = Color.HSVToRGB(h, s, v); }
    	private void SetColorPeaks(float h, float s, float v) { colorPeaksRGB = Color.HSVToRGB(h, s, v); }

		//////////////////////////////////////////////////////////////////////////////////
		// AVD Synchronization - called by SendMessage from Master
		//////////////////////////////////////////////////////////////////////////////////
		public void Sync(float[] syncdata)
		{
			try
			{
				avd.UnpackSyncdata(syncdata);
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}
	}
}