﻿// ***********************************************************************************
// SallySoundFX
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// https://www.patreon.com/sallyw
// Licensed under CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/

using System;
using System.Text;
using UnityEngine.UI;

namespace Sally
{
    public class AVSLAVE_Debug : MVRScript
	{
		StringBuilder sb = new StringBuilder();
		UIDynamicButton buttonAVD;
		private AVD avd = new AVD();

		public override void Init()
		{
			try
			{
				if (AVSlave.GetMasterScript(this) == null)
					SuperController.LogMessage($"{name} could not find SallySoundFX Plugin to synchronize. It will retry automatically.");

				UIDynamicButton button = CreateButton("Audio Visualization Data (AVD) from SallySoundFX:");
				button.GetComponent<LayoutElement>().minWidth = 1060;
				buttonAVD = CreateButton("avd.bands data");
				buttonAVD.height = 1100;
				buttonAVD.GetComponent<LayoutElement>().minWidth = 1060;
				buttonAVD.button.enabled = false;
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		// Update is called with each rendered frame by Unity
		void Update()
		{
			try
			{
				if (!AVSlave.IsSynchronized(this))
					return;

				// bands
				sb.Length = 0;
				sb.Append("avd.bands = {");
				foreach (float f in avd.bands)
					sb.Append(f.ToString("0.00")).Append(", ");

				// bands_peak
				sb.Append("}\n\navd.bands_peak = {");
				foreach (float f in avd.bands_peak)
					sb.Append(f.ToString("0.00")).Append(", ");

				// samples
				sb.Append("}\n\navd.samples = {");
				foreach (float f in avd.samples)
					sb.Append(f.ToString("0.00")).Append(", ");

				// samples_peak
				sb.Append("}\n\navd.samples_peak = {");
				foreach (float f in avd.samples_peak)
					sb.Append(f.ToString("0.00")).Append(", ");

				// indexTrigger1
				sb.Append("}\n\navd.indexTrigger1 = ").Append((int)avd.indexTrigger1);

				// indexTrigger2
				sb.Append("\navd.indexTrigger2 = ").Append((int)avd.indexTrigger2);

				// version
				sb.Append("\navd.version = ").Append(avd.version.ToString("0.0000"));

				buttonAVD.buttonText.text = sb.ToString();
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		//////////////////////////////////////////////////////////////////////////////////
		// AVD Synchronization - called by SendMessage from Master
		//////////////////////////////////////////////////////////////////////////////////
		public void Sync(float[] syncdata)
		{
			try
			{
				avd.UnpackSyncdata(syncdata);
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}
	}
}
