﻿// ***********************************************************************************
// SallySoundFX
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// https://www.patreon.com/sallyw
// Licensed under CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/

using System;
using UnityEngine.UI;
using MacGruber;
using UnityEngine;
using SimpleJSON;

namespace Sally
{
    public class AVSLAVE_MoreTriggers : MVRScript
    {
        // MISC
        private UISpectrum uiSpectrum;
        private GameObject goSpectrum;

        // UI - LEFT - Trigger 3
        private JSONStorableFloat jfTrigger3Freq;
        private FloatTrigger fTrigger3;
        private int indexTrigger3;

        // UI - LEFT - Trigger 4
        private JSONStorableFloat jfTrigger4Freq;
        private FloatTrigger fTrigger4;
        private int indexTrigger4;

        // UI - LEFT - Trigger 5
        private JSONStorableFloat jfTrigger5Freq;
        private FloatTrigger fTrigger5;
        private int indexTrigger5;

        // UI - LEFT - Trigger 6
        private JSONStorableFloat jfTrigger6Freq;
        private FloatTrigger fTrigger6;
        private int indexTrigger6;

        // UI - LEFT - Trigger 7
        private JSONStorableFloat jfTrigger7Freq;
        private FloatTrigger fTrigger7;
        private int indexTrigger7;

        // UI - LEFT - Trigger 8
        private JSONStorableFloat jfTrigger8Freq;
        private FloatTrigger fTrigger8;
        private int indexTrigger8;

        // UI - LEFT - Trigger 9
        private JSONStorableFloat jfTrigger9Freq;
        private FloatTrigger fTrigger9;
        private int indexTrigger9;

        // UI - LEFT - Trigger 10
        private JSONStorableFloat jfTrigger10Freq;
        private FloatTrigger fTrigger10;
        private int indexTrigger10;

        // UI - RIGHT
        private JSONStorableBool jbZoomSpectrum;

        // AUDIO VISUALIZATION DATA
        private AVD avd = new AVD();

        // Color used when slider is value is modified
        private static Color lastSelectedColor = Color.green;
        // Color used to reset slider back to default
        private static Color sliderDefaultColor = Color.white;
        // Colors used to help visually separate triggers with different button- & slider backgrounds
        private static Color separateBGColorButton = new Color(0.7f, 0.7f, 0.7f);
        private static Color separateBGColorSlider = new Color(0.8f, 0.8f, 0.8f);

        public override void Init()
        {
            try
            {
                if (AVSlave.GetMasterScript(this) == null)
                    SuperController.LogMessage($"{name} could not find SallySoundFX Plugin to synchronize. It will retry automatically.");

                //////////////////////////////////////////////////////////////////////////////////
                // UI - TOP
                //////////////////////////////////////////////////////////////////////////////////

                UIDynamic spacer = CreateSpacer();
                goSpectrum = new GameObject("Spectrum");
                goSpectrum.transform.SetParent(spacer.transform, false);
                uiSpectrum = goSpectrum.AddComponent<UISpectrum>();
                uiSpectrum.showBackground = true;
                uiSpectrum.trigger1Color = lastSelectedColor;
                uiSpectrum.transform.position = new Vector2(-250, -990);
                uiSpectrum.sampleIndexTrigger1 = 1; //indexTrigger1;
                uiSpectrum.sampleIndexTrigger2 = -1;

                CreateSpacer(false).height = 175;
                UIDynamicTextField hintText = CreateTextField(new JSONStorableString("Hint", "<size=25><b><color=#C0C0C0>Green bar is the last modified frequency.</color></b></size>"), true);
                hintText.GetComponentInChildren<Image>().color = Color.clear;
                CreateSpacer(true).height = 75; // 290;

                //////////////////////////////////////////////////////////////////////////////////
                // UI - LEFT
                //////////////////////////////////////////////////////////////////////////////////
                UIDynamicSlider slider;
                UIDynamicButton button;

                // Button Trigger 3
                fTrigger3 = new FloatTrigger(this, "Trigger 3");
                button = CreateButton("Trigger 3 Actions", false);
                //button.buttonText.color = trigger1Color;
                button.button.onClick.AddListener(fTrigger3.OpenPanel);
                // Slider Trigger 3
                jfTrigger3Freq = new JSONStorableFloat("Trigger 3 Frequency in Hz", AVD.SampleFreq() * 2, AVD.SampleFreq(), AVD.SampleFreq() * AVD.SAMPLES * 0.5f);
                RegisterFloat(jfTrigger3Freq);
                slider = CreateSlider(jfTrigger3Freq, false);
                //slider.labelText.color = trigger1Color;
                slider.quickButtonsEnabled = false;
                slider.defaultButtonEnabled = true;
                // Set Trigger 4 as default lastSelected
                slider.sliderControl.slider.image.color = lastSelectedColor; 
                slider.GetComponent<LayoutElement>().minWidth = 1040;
                jfTrigger3Freq.setCallbackFunction = SetTrigger3Frequency;
                fTrigger3.Name = "Trigger 3: " + AVD.SampleFreq() * 2 + " Hz";

                // Button Trigger 4
                fTrigger4 = new FloatTrigger(this, "Trigger 4");
                button = CreateButton("Trigger 4 Actions", false);
                button.GetComponentInChildren<Image>().color = separateBGColorButton;
                button.button.onClick.AddListener(fTrigger4.OpenPanel);
                // Slider Trigger 4
                jfTrigger4Freq = new JSONStorableFloat("Trigger 4 Frequency in Hz", AVD.SampleFreq() * 4, AVD.SampleFreq(), AVD.SampleFreq() * AVD.SAMPLES * 0.5f);
                RegisterFloat(jfTrigger4Freq);
                slider = CreateSlider(jfTrigger4Freq, false);
                slider.quickButtonsEnabled = false;
                slider.defaultButtonEnabled = true;
                slider.GetComponent<LayoutElement>().minWidth = 1040;
                slider.GetComponentInChildren<Image>().color = separateBGColorSlider;
                jfTrigger4Freq.setCallbackFunction = SetTrigger4Frequency;
                fTrigger4.Name = "Trigger 4: " + AVD.SampleFreq() * 4 + " Hz";

                // Button Trigger 5
                fTrigger5 = new FloatTrigger(this, "Trigger 5");
                button = CreateButton("Trigger 5 Actions", false);
                button.button.onClick.AddListener(fTrigger5.OpenPanel);
                // Slider Trigger 5
                jfTrigger5Freq = new JSONStorableFloat("Trigger 5 Frequency in Hz", AVD.SampleFreq() * 6, AVD.SampleFreq(), AVD.SampleFreq() * AVD.SAMPLES * 0.5f);
                RegisterFloat(jfTrigger5Freq);
                slider = CreateSlider(jfTrigger5Freq, false);
                slider.quickButtonsEnabled = false;
                slider.defaultButtonEnabled = true;
                slider.GetComponent<LayoutElement>().minWidth = 1040;
                jfTrigger5Freq.setCallbackFunction = SetTrigger5Frequency;
                fTrigger5.Name = "Trigger 5: " + AVD.SampleFreq() * 6 + " Hz";

                // Button Trigger 6
                fTrigger6 = new FloatTrigger(this, "Trigger 6");
                button = CreateButton("Trigger 6 Actions", false);
                button.GetComponentInChildren<Image>().color = separateBGColorButton;
                button.button.onClick.AddListener(fTrigger6.OpenPanel);
                // Slider Trigger 6
                jfTrigger6Freq = new JSONStorableFloat("Trigger 6 Frequency in Hz", AVD.SampleFreq() * 8, AVD.SampleFreq(), AVD.SampleFreq() * AVD.SAMPLES * 0.5f);
                RegisterFloat(jfTrigger6Freq);
                slider = CreateSlider(jfTrigger6Freq, false);
                slider.quickButtonsEnabled = false;
                slider.defaultButtonEnabled = true;
                slider.GetComponent<LayoutElement>().minWidth = 1040;
                slider.GetComponentInChildren<Image>().color = separateBGColorSlider;
                jfTrigger6Freq.setCallbackFunction = SetTrigger6Frequency;
                fTrigger6.Name = "Trigger 6: " + AVD.SampleFreq() * 8 + " Hz";

                // Button Trigger 7
                fTrigger7 = new FloatTrigger(this, "Trigger 7");
                button = CreateButton("Trigger 7 Actions", false);
                button.button.onClick.AddListener(fTrigger7.OpenPanel);
                // Slider Trigger 7
                jfTrigger7Freq = new JSONStorableFloat("Trigger 7 Frequency in Hz", AVD.SampleFreq() * 10, AVD.SampleFreq(), AVD.SampleFreq() * AVD.SAMPLES * 0.5f);
                RegisterFloat(jfTrigger7Freq);
                slider = CreateSlider(jfTrigger7Freq, false);
                slider.quickButtonsEnabled = false;
                slider.defaultButtonEnabled = true;
                slider.GetComponent<LayoutElement>().minWidth = 1040;
                jfTrigger7Freq.setCallbackFunction = SetTrigger7Frequency;
                fTrigger7.Name = "Trigger 7: " + AVD.SampleFreq() * 10 + " Hz";

                // Button Trigger 8
                fTrigger8 = new FloatTrigger(this, "Trigger 8");
                button = CreateButton("Trigger 8 Actions", false);
                button.GetComponentInChildren<Image>().color = separateBGColorButton;
                button.button.onClick.AddListener(fTrigger8.OpenPanel);
                // Slider Trigger 8
                jfTrigger8Freq = new JSONStorableFloat("Trigger 8 Frequency in Hz", AVD.SampleFreq() * 12, AVD.SampleFreq(), AVD.SampleFreq() * AVD.SAMPLES * 0.5f);
                RegisterFloat(jfTrigger8Freq);
                slider = CreateSlider(jfTrigger8Freq, false);
                slider.quickButtonsEnabled = false;
                slider.defaultButtonEnabled = true;
                slider.GetComponentInChildren<Image>().color = separateBGColorSlider;
                slider.GetComponent<LayoutElement>().minWidth = 1040;
                jfTrigger8Freq.setCallbackFunction = SetTrigger8Frequency;
                fTrigger8.Name = "Trigger 8: " + AVD.SampleFreq() * 12 + " Hz";

                // Button Trigger 9
                fTrigger9 = new FloatTrigger(this, "Trigger 9");
                button = CreateButton("Trigger 9 Actions", false);
                button.button.onClick.AddListener(fTrigger9.OpenPanel);
                // Slider Trigger 9
                jfTrigger9Freq = new JSONStorableFloat("Trigger 9 Frequency in Hz", AVD.SampleFreq() * 14, AVD.SampleFreq(), AVD.SampleFreq() * AVD.SAMPLES * 0.5f);
                RegisterFloat(jfTrigger9Freq);
                slider = CreateSlider(jfTrigger9Freq, false);
                slider.quickButtonsEnabled = false;
                slider.defaultButtonEnabled = true;
                slider.GetComponent<LayoutElement>().minWidth = 1040;
                jfTrigger9Freq.setCallbackFunction = SetTrigger9Frequency;
                fTrigger9.Name = "Trigger 9: " + AVD.SampleFreq() * 14 + " Hz";

                // Button Trigger 10
                fTrigger10 = new FloatTrigger(this, "Trigger 10");
                button = CreateButton("Trigger 10 Actions", false);
                button.GetComponentInChildren<Image>().color = separateBGColorButton;
                button.button.onClick.AddListener(fTrigger10.OpenPanel);
                // Slider Trigger 10
                jfTrigger10Freq = new JSONStorableFloat("Trigger 10 Frequency in Hz", AVD.SampleFreq() * 16, AVD.SampleFreq(), AVD.SampleFreq() * AVD.SAMPLES * 0.5f);
                RegisterFloat(jfTrigger10Freq);
                slider = CreateSlider(jfTrigger10Freq, false);
                slider.quickButtonsEnabled = false;
                slider.defaultButtonEnabled = true;
                slider.GetComponentInChildren<Image>().color = separateBGColorSlider;
                slider.GetComponent<LayoutElement>().minWidth = 1040;
                jfTrigger10Freq.setCallbackFunction = SetTrigger10Frequency;
                fTrigger10.Name = "Trigger 10: " + AVD.SampleFreq() * 16 + " Hz";

                //////////////////////////////////////////////////////////////////////////////////
                // RIGHT
                //////////////////////////////////////////////////////////////////////////////////

                // Toggle Zoom
                jbZoomSpectrum = new JSONStorableBool("Zoom Spectrum to 64 samples", true);
                RegisterBool(jbZoomSpectrum);
                CreateToggle(jbZoomSpectrum, true);
                jbZoomSpectrum.setCallbackFunction = SetSpectrumSize;

                // required for MacGruber_utils
                SimpleTriggerHandler.LoadAssets();
            }
            catch (Exception e) { SuperController.LogError("Exception: " + e); }
        }

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
                jc[fTrigger3.Name] = fTrigger3.GetJSON(base.subScenePrefix);
                jc[fTrigger4.Name] = fTrigger4.GetJSON(base.subScenePrefix);
                jc[fTrigger5.Name] = fTrigger5.GetJSON(base.subScenePrefix);
                jc[fTrigger6.Name] = fTrigger6.GetJSON(base.subScenePrefix);
                jc[fTrigger7.Name] = fTrigger7.GetJSON(base.subScenePrefix);
                jc[fTrigger8.Name] = fTrigger8.GetJSON(base.subScenePrefix);
                jc[fTrigger9.Name] = fTrigger9.GetJSON(base.subScenePrefix);
                jc[fTrigger10.Name] = fTrigger10.GetJSON(base.subScenePrefix);
            }
            return jc;
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                fTrigger3.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                fTrigger4.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                fTrigger5.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                fTrigger6.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                fTrigger7.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                fTrigger8.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                fTrigger9.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                fTrigger10.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            try
            {
                if (!AVSlave.IsSynchronized(this))
                    return;

                // Set sample values for Frequency Triggers based on user selected array index
                fTrigger3.Trigger(avd.samples[indexTrigger3]);
                fTrigger4.Trigger(avd.samples[indexTrigger4]);
                fTrigger5.Trigger(avd.samples[indexTrigger5]);
                fTrigger6.Trigger(avd.samples[indexTrigger6]);
                fTrigger7.Trigger(avd.samples[indexTrigger7]);
                fTrigger8.Trigger(avd.samples[indexTrigger8]);
                fTrigger9.Trigger(avd.samples[indexTrigger9]);
                fTrigger10.Trigger(avd.samples[indexTrigger10]);

                // Update Plugin UI Spectrum
                uiSpectrum.UpdateSamples(avd);
            }
            catch (Exception e) { SuperController.LogError("Exception: " + e); }
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy()
        {
            try
            {
                avd = null;

                if (fTrigger3 != null) { fTrigger3.Remove(); fTrigger3 = null; }
                if (fTrigger4 != null) { fTrigger4.Remove(); fTrigger4 = null; }
                if (fTrigger5 != null) { fTrigger5.Remove(); fTrigger5 = null; }
                if (fTrigger6 != null) { fTrigger6.Remove(); fTrigger6 = null; }
                if (fTrigger7 != null) { fTrigger7.Remove(); fTrigger7 = null; }
                if (fTrigger8 != null) { fTrigger8.Remove(); fTrigger8 = null; }
                if (fTrigger9 != null) { fTrigger9.Remove(); fTrigger9 = null; }
                if (fTrigger10 != null) { fTrigger10.Remove(); fTrigger10 = null; }

                uiSpectrum = null;

                // UI Spectrum
                if (goSpectrum != null)
                {
                    Destroy(goSpectrum);
                    goSpectrum = null;
                }
            }
            catch (Exception e) { SuperController.LogError("Exception: " + e); }
        }

        //////////////////////////////////////////////////////////////////////////////////
        // UI Callbacks
        //////////////////////////////////////////////////////////////////////////////////
        private void SetSpectrumSize(bool b)
        {
            if (b)
            {
                uiSpectrum.size = 64;
                jfTrigger3Freq.max = AVD.SampleFreq() * AVD.SAMPLES * 0.5f;
                jfTrigger4Freq.max = AVD.SampleFreq() * AVD.SAMPLES * 0.5f;
                jfTrigger5Freq.max = AVD.SampleFreq() * AVD.SAMPLES * 0.5f;
                jfTrigger6Freq.max = AVD.SampleFreq() * AVD.SAMPLES * 0.5f;
                jfTrigger7Freq.max = AVD.SampleFreq() * AVD.SAMPLES * 0.5f;
                jfTrigger8Freq.max = AVD.SampleFreq() * AVD.SAMPLES * 0.5f;
                jfTrigger9Freq.max = AVD.SampleFreq() * AVD.SAMPLES * 0.5f;
                jfTrigger10Freq.max = AVD.SampleFreq() * AVD.SAMPLES * 0.5f;
            }
            else
            {
                uiSpectrum.size = 128;
                jfTrigger3Freq.max = AVD.SampleFreq() * AVD.SAMPLES;
                jfTrigger4Freq.max = AVD.SampleFreq() * AVD.SAMPLES;
                jfTrigger5Freq.max = AVD.SampleFreq() * AVD.SAMPLES;
                jfTrigger6Freq.max = AVD.SampleFreq() * AVD.SAMPLES;
                jfTrigger7Freq.max = AVD.SampleFreq() * AVD.SAMPLES;
                jfTrigger8Freq.max = AVD.SampleFreq() * AVD.SAMPLES;
                jfTrigger9Freq.max = AVD.SampleFreq() * AVD.SAMPLES;
                jfTrigger10Freq.max = AVD.SampleFreq() * AVD.SAMPLES;
            }
        }

        private void SetTrigger3Frequency(float f)
        {
            f = AVD.SnapToSampleFrequency(f);
            jfTrigger3Freq.val = f;
            indexTrigger3 = (int)(f / AVD.SampleFreq()) - 1;
            uiSpectrum.sampleIndexTrigger1 = indexTrigger3;
            fTrigger3.Name = "Trigger 3: " + f + " Hz";
            ResetAllSliderColors();
            jfTrigger3Freq.slider.image.color = lastSelectedColor;
        }

        private void SetTrigger4Frequency(float f)
        {
            f = AVD.SnapToSampleFrequency(f);
            jfTrigger4Freq.val = f;
            indexTrigger4 = (int)(f / AVD.SampleFreq()) - 1;
            uiSpectrum.sampleIndexTrigger1 = indexTrigger4;
            fTrigger4.Name = "Trigger 4: " + f + " Hz";
            ResetAllSliderColors();
            jfTrigger4Freq.slider.image.color = lastSelectedColor;
        }

        private void SetTrigger5Frequency(float f)
        {
            f = AVD.SnapToSampleFrequency(f);
            jfTrigger5Freq.val = f;
            indexTrigger5 = (int)(f / AVD.SampleFreq()) - 1;
            uiSpectrum.sampleIndexTrigger1 = indexTrigger5;
            fTrigger5.Name = "Trigger 5: " + f + " Hz";
            ResetAllSliderColors();
            jfTrigger5Freq.slider.image.color = lastSelectedColor;
        }

        private void SetTrigger6Frequency(float f)
        {
            f = AVD.SnapToSampleFrequency(f);
            jfTrigger6Freq.val = f;
            indexTrigger6 = (int)(f / AVD.SampleFreq()) - 1;
            uiSpectrum.sampleIndexTrigger1 = indexTrigger6;
            fTrigger6.Name = "Trigger 6: " + f + " Hz";
            ResetAllSliderColors();
            jfTrigger6Freq.slider.image.color = lastSelectedColor;
        }

        private void SetTrigger7Frequency(float f)
        {
            f = AVD.SnapToSampleFrequency(f);
            jfTrigger7Freq.val = f;
            indexTrigger7 = (int)(f / AVD.SampleFreq()) - 1;
            uiSpectrum.sampleIndexTrigger1 = indexTrigger7;
            fTrigger7.Name = "Trigger 7: " + f + " Hz";
            ResetAllSliderColors();
            jfTrigger7Freq.slider.image.color = lastSelectedColor;
        }

        private void SetTrigger8Frequency(float f)
        {
            f = AVD.SnapToSampleFrequency(f);
            jfTrigger8Freq.val = f;
            indexTrigger8 = (int)(f / AVD.SampleFreq()) - 1;
            uiSpectrum.sampleIndexTrigger1 = indexTrigger8;
            fTrigger8.Name = "Trigger 8: " + f + " Hz";
            ResetAllSliderColors();
            jfTrigger8Freq.slider.image.color = lastSelectedColor;
        }

        private void SetTrigger9Frequency(float f)
        {
            f = AVD.SnapToSampleFrequency(f);
            jfTrigger9Freq.val = f;
            indexTrigger9 = (int)(f / AVD.SampleFreq()) - 1;
            uiSpectrum.sampleIndexTrigger1 = indexTrigger9;
            fTrigger9.Name = "Trigger 9: " + f + " Hz";
            ResetAllSliderColors();
            jfTrigger9Freq.slider.image.color = lastSelectedColor;
        }

        private void SetTrigger10Frequency(float f)
        {
            f = AVD.SnapToSampleFrequency(f);
            jfTrigger10Freq.val = f;
            indexTrigger10 = (int)(f / AVD.SampleFreq()) - 1;
            uiSpectrum.sampleIndexTrigger1 = indexTrigger10;
            fTrigger10.Name = "Trigger 10: " + f + " Hz";
            ResetAllSliderColors();
            jfTrigger10Freq.slider.image.color = lastSelectedColor;
        }

        //////////////////////////////////////////////////////////////////////////////////
        // Helper functions
        //////////////////////////////////////////////////////////////////////////////////
        private void ResetAllSliderColors()
        {
            jfTrigger3Freq.slider.image.color = sliderDefaultColor;
            jfTrigger4Freq.slider.image.color = sliderDefaultColor;
            jfTrigger5Freq.slider.image.color = sliderDefaultColor;
            jfTrigger6Freq.slider.image.color = sliderDefaultColor;
            jfTrigger7Freq.slider.image.color = sliderDefaultColor;
            jfTrigger8Freq.slider.image.color = sliderDefaultColor;
            jfTrigger9Freq.slider.image.color = sliderDefaultColor;
            jfTrigger10Freq.slider.image.color = sliderDefaultColor;
        }

        //////////////////////////////////////////////////////////////////////////////////
        // AVD Synchronization - called by SendMessage from Master
        //////////////////////////////////////////////////////////////////////////////////
        public void Sync(float[] syncdata)
        {
            try
            {
                avd.UnpackSyncdata(syncdata);
            }
            catch (Exception e) { SuperController.LogError("Exception: " + e); }
        }
    }
}
