﻿// ***********************************************************************************
// SallySoundFX
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// https://www.patreon.com/sallyw
// Licensed under CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/

using System;
using UnityEngine;
using UnityEngine.Profiling;

namespace Sally
{
    // Audio Visualization Data
    public class AVD
    {
        public static readonly string MASTER_PLUGIN_IDSTRING = "AVMASTER";
        public static readonly string SLAVE_PLUGIN_IDSTRING = "AVSLAVE";
        public static readonly float VERSION = 2.1f;
        
        // Unity sample rate: 48000 Hz
        // Samples requested from Unity: 512
        // Samples used by Plugin: first 128
        // Plugin frequency range: 46.875 Hz to 6000 Hz
        public const int SAMPLES_UNITY = 512;    // sample size requested from Unity
        public const int SAMPLES = 128;          // sample size actually used for Plugin
        public const int BANDS = 8;              // number of frequency bands
        public static readonly Color trigger1Color = new Color(0.7f, 0f, 0f, 1.0f);
        public static readonly Color trigger2Color = new Color(0f, 0.4f, 1f, 1.0f);
        private static float samplefreq = (float)AudioSettings.outputSampleRate / SAMPLES_UNITY * 0.5f;

        public float[] bands = new float[BANDS];           // 8 frequency bands
        public float[] bands_peak = new float[BANDS];      // 8 peak values of bands[]
        public float[] samples = new float[SAMPLES];       // 128 samples
        public float[] samples_peak = new float[SAMPLES];  // 128 peak values of samples[]
        public int indexTrigger1 = 2;                      // samples[indexTrigger1] for SSFX Trigger 1
        public int indexTrigger2 = 15;                     // samples[indexTrigger2] for SSFX Trigger 2
        public float version = 0f;
        private float[] syncdata = new float[BANDS + BANDS + SAMPLES + SAMPLES + 3];

        public static float SampleFreq() { return samplefreq; }
        public static float SnapToSampleFrequency(float v) {return (float)Math.Round(v / samplefreq) * samplefreq;}

        private int isync; // here for optimization
        public float[] PackSyncdata()
        {
            isync = 0; // index for syncdata array
            int i = 0;
            for (i = 0; i < bands.Length; i++)
            {
                syncdata[isync] = bands[i];
                isync++;
            }
            for (i = 0; i < bands_peak.Length; i++)
            {
                syncdata[isync] = bands_peak[i];
                isync++;
            }
            Array.Copy(samples, 0, syncdata, isync, SAMPLES);
            isync += SAMPLES;
            Array.Copy(samples_peak, 0, syncdata, isync, SAMPLES);
            isync += SAMPLES;
            syncdata[isync] = (float)indexTrigger1;
            isync++;
            syncdata[isync] = (float)indexTrigger2;
            isync++;
            syncdata[isync] = VERSION;
            return syncdata;
        }

        public void UnpackSyncdata(float[] syncdata)
        {
            isync = 0; // index for syncdata array
            int i = 0;
            for (i = 0; i < bands.Length; i++)
            {
                bands[i] = syncdata[isync];
                isync++;
            }
            for (i = 0; i < bands_peak.Length; i++)
            {
                bands_peak[i] = syncdata[isync];
                isync++;
            }
            Array.Copy(syncdata, isync, samples, 0, SAMPLES);
            isync += SAMPLES;
            Array.Copy(syncdata, isync, samples_peak, 0, SAMPLES);
            isync += SAMPLES;
            indexTrigger1 = (int)syncdata[isync];
            isync++;
            indexTrigger2 = (int)syncdata[isync];
            isync++;
            version = syncdata[isync];
        }
    }
}