﻿// ***********************************************************************************
// SallySoundFX
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// https://www.patreon.com/sallyw
// Licensed under CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/

using UnityEngine;

namespace Sally
{
	// Look Up Table
    public static class LUT
    {
		// 2x positive sinus waves
		public static float[] sin2x = CreateSinWave(AVD.SAMPLES, 2, true);

		// Generate Sinus LUT to avoid low performance Mathf.Sin() at runtime
		public static float[] CreateSinWave(int arraySize, int waveCount, bool positiveOnly)
		{
			float[] wave = new float[arraySize];
			float step = (waveCount * Mathf.PI * 2) / (float)arraySize;
			float shift = 0;
			if (positiveOnly)
				shift = 1.5f * Mathf.PI; // start wave at zero
			for (int i = 0; i < arraySize; i++)
			{
				if (positiveOnly)
					wave[i] = 0.5f + 0.5f * Mathf.Sin(step * (i + 1) + shift);
				else
					wave[i] = Mathf.Sin(step * (i + 1));
				//Debug.Log(i + " " + wave[i].ToString("0.000"));
			}
			return wave;
		}
	}
}
