﻿// ***********************************************************************************
// SallySoundFX
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// https://www.patreon.com/sallyw
// Licensed under CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/

using UnityEngine;

namespace Sally
{
    static class Mic
    {
		private static bool active = false;
		private static int id = -1;
		private static string device = null;
		private static int minFreq = 0;
		private static int maxFreq = 0;
		private static AudioSource micAudioSource = null;

		public static void ListDevices()
        {
			if (Microphone.devices.Length == 0)
			{
				SuperController.LogMessage("SallySoundFX No microphone found.");
				return;
			}

			// Get list of Microphone devices and print the names to the log
			int minFreq, maxFreq;
			for (int i = 0; i < Microphone.devices.Length; i++)
			{
				Microphone.GetDeviceCaps(Microphone.devices[i], out minFreq, out maxFreq);
				SuperController.LogMessage("SallySoundFX Microphone id: " + i + " " + Microphone.devices[i] + " " + maxFreq + " Hz");
			}
		}

		public static void SelectByID(int id)
        {
			if (active)
            {
				SuperController.LogMessage("SallySoundFX ERROR: Microphone id cannot be selected while active.");
				return;
            }

			if (id >= Microphone.devices.Length)
            {
				Mic.active = false;
				Mic.id = -1;
				Mic.device = null;
				Mic.minFreq = 0;
				Mic.maxFreq = 0;
				SuperController.LogMessage("SallySoundFX ERROR: Microphone id invalid!");
				return;
			}
			Mic.id = id;
			Mic.device = Microphone.devices[id];
			Microphone.GetDeviceCaps(Microphone.devices[id], out minFreq, out maxFreq);
		}

		public static void On()
		{
			if (id == -1)
			{
				SuperController.LogMessage("SallySoundFX ERROR: Microphone id not selected!");
				return;
			}
			GameObject goMicrophoneAudioSource = GameObject.Find("AudioSourceMicrophone");
			if (goMicrophoneAudioSource == null)
			{
				SuperController.LogMessage("SallySoundFX To use a microphone rename an \"AudioSource\" in the scene to \"AudioSourceMicrophone\"");
				return;
			}
			micAudioSource = goMicrophoneAudioSource.GetComponentInChildren<AudioSource>();//GetComponent<AudioSource>();
			if (micAudioSource == null)
			{
				SuperController.LogMessage("SallySoundFX ERROR: Could not get AudioSource from GameObject!");
				return;
			}

			micAudioSource.clip = Microphone.Start(Microphone.devices[id], true, 1, 48000);
			if (micAudioSource.clip == null)
			{
				micAudioSource = null;
				SuperController.LogMessage("SallySoundFX ERROR: Microphone.Start() failed to start recording!");
				return;
			}
			// https://support.unity.com/hc/en-us/articles/206485253-How-do-I-get-Unity-to-playback-a-Microphone-input-in-real-time-
			micAudioSource.loop = true;
			while (!(Microphone.GetPosition(null) > 0)) { }
			micAudioSource.Play();
			active = true;
			SuperController.LogMessage("SallySoundFX Microphone ON! Make sure VAM has permission to use it.");
		}

		public static void Off()
        {
			if (device == null)
				return;
			Microphone.End(device);
			if (micAudioSource != null)
				micAudioSource.Stop();
			active = false;
			SuperController.LogMessage("SallySoundFX Microphone OFF!");
		}
	}
}
