﻿// ***********************************************************************************
// SallySoundFX
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// https://www.patreon.com/sallyw
// Licensed under CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/

using UnityEngine;
using UnityEngine.UI;

namespace Sally
{
	public class UISpectrum : Graphic
	{
		private float _width = 1024;// 128 x 8 bar width = 1024
		private float _height = 280;
		private float border = 10;
		private float[] bands; // keep to re-add band spectrum!
		private float[] bands_peak; // keep to re-add band spectrum!
		private float[] samples;
		private float[] samples_peak;
		//private float[] bands_amp;  // keep?
		public int sampleIndexTrigger1 = 1;
		public int sampleIndexTrigger2 = 15;
		public int size = 64;
		public bool showBands = false;
		public bool showBackground = false;
        // TRIGGER COLORS
		public Color trigger1Color = AVD.trigger1Color;
        public Color trigger2Color = AVD.trigger2Color;

		// OPTIMIZATION
		private int i;
		private int vIndex;
		private float barWidth;
		private float sampleValue;
		private float sampleValuePeakXHeight;
		private float iXbarWidth;
		private Color32 peakColor = Color.white;

		public float Width
		{
			get { return _width; }
			set { _width = value; SetVerticesDirty(); }
		}

		public float Height
		{
			get { return _height; }
			set { _height = value; SetVerticesDirty(); }
		}

		protected override void OnPopulateMesh(VertexHelper vh)
		{
			vh.Clear();
			UpdateBars(vh);
		}

		// 0.021 - 0.022 ms / call
		private void UpdateBars(VertexHelper vh)
		{
			UIVertex vertex = UIVertex.simpleVert;

			////////////////////////////////////////////
			// 2.02 new lines
			////////////////////////////////////////////

			// black background
			if (showBackground)
				vertex.color = Color.black;
			else
				vertex.color = Color.clear;

            // bottom left
            vertex.position.x = -border;
            vertex.position.y = -border;
            vh.AddVert(vertex);
            // top left
            vertex.position.x = -border;
            vertex.position.y = border + _height;
            vh.AddVert(vertex);
            // top right
            vertex.position.x = border + _width;
            vertex.position.y = border + _height;
            vh.AddVert(vertex);
            // bottom right
            vertex.position.x = border + _width;
            vertex.position.y = -border;
            vh.AddVert(vertex);

            // Build triangles from vertex indices
            vh.AddTriangle(0, 1, 2); // triangle 1
            vh.AddTriangle(2, 3, 0); // triangle 2
            vIndex = 4;

			// reset alpha on first spectrum bar
			vertex.color.a = 255;

            ////////////////////////////////////////////
            // 2.02 new lines end
            ////////////////////////////////////////////
            // vIndex = 0; // old
            barWidth = _width / size;
			for (i = 0; i < size; i++)
			{
				// Optimization
				sampleValue = samples[i];
				sampleValuePeakXHeight = samples_peak[i] * _height;
				iXbarWidth = i * barWidth;

				// BARS
				vertex.color.r = (byte)(Mathf.Clamp01(sampleValue) * 255f);
				vertex.color.g = (byte)(Mathf.Clamp01(1f - sampleValue) * 255f);
				vertex.color.b = 255; //(byte)(Mathf.Clamp01(1f) * 255f);
				if (i == sampleIndexTrigger1)
					vertex.color = trigger1Color;
                if (i == sampleIndexTrigger2)
					vertex.color = trigger2Color;
                // bottom left
                vertex.position.x = iXbarWidth;
				vertex.position.y = 0;
				vh.AddVert(vertex);
				// top left
				vertex.position.x = iXbarWidth;
				vertex.position.y = sampleValue * _height;
				vh.AddVert(vertex);
				// top right
				vertex.position.x = iXbarWidth + barWidth;
				vertex.position.y = sampleValue * _height;
				vh.AddVert(vertex);
				// bottom right
				vertex.position.x = iXbarWidth + barWidth;
				vertex.position.y = 0;
				vh.AddVert(vertex);

				// PEAK TRIANGLES
				vertex.color = peakColor;
				// bottom left
				vertex.position.x = iXbarWidth;
				vertex.position.y = sampleValuePeakXHeight;
				vh.AddVert(vertex);
				// center top
				vertex.position.x = iXbarWidth + barWidth * 0.5f;
				vertex.position.y = sampleValuePeakXHeight + 8f;
				vh.AddVert(vertex);
				// bottom right
				vertex.position.x = iXbarWidth + barWidth;
				vertex.position.y = sampleValuePeakXHeight;
				vh.AddVert(vertex);

				// Build triangles from vertex indices
				vIndex = 4 + i * 7; // old: vIndex = i * 7;
                vh.AddTriangle(vIndex, vIndex + 1, vIndex + 2);     // bar triangle 1
				vh.AddTriangle(vIndex + 2, vIndex + 3, vIndex);     // bar triangle 2
				vh.AddTriangle(vIndex + 4, vIndex + 5, vIndex + 6); // peak triangle
			}
		}

		public void UpdateSamples(AVD avd) //, float[] bands_amp)
		{
			bands = avd.bands;
			bands_peak = avd.bands_peak;
			samples = avd.samples;
			samples_peak = avd.samples_peak;
			//this.bands_amp = bands_amp;
			SetVerticesDirty();
		}
	}
}

// OLD CODE - 0.03 ms / call
/*
		private void UpdateBars(VertexHelper vh)
		{
			UIVertex vertex = UIVertex.simpleVert;

			int vIndex = 0;
			if (showBands)
            {
				float largestBandPPP = 0f;
				for (int i = 0; i < bands_amp.Length; i++)
					if (bands_amp[i] > largestBandPPP)
						largestBandPPP = bands_amp[i];
				float bandPPPtoHeightScale = largestBandPPP / _height;

				float barWidth = _width / bands.Length;
				for (int i = 0; i < bands.Length; i++)
				{
					vertex.color = new Color(1f * bands[i], 1f - bands[i], 1f, 1.0f);
					vertex.position = new Vector3(i * barWidth, 0);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth, bands[i] * _height);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth, bands[i] * _height);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth, 0);
					vh.AddVert(vertex);

					// Push Pull Peaks
					vertex.color = Color.green;
					vertex.position = new Vector3(i * barWidth, bands_peak[i] * _height);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth * 0.5f, bands_peak[i] * _height + 4f);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth * 0.5f, bands_peak[i] * _height - 4f);
					vh.AddVert(vertex);

					// Band Amp
					vertex.color = Color.red;
					float ampY = _height * bandPPPtoHeightScale * bands_amp[i];
					vertex.position = new Vector3(i * barWidth, ampY);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth * 0.5f, ampY + 8f);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth, ampY);
					vh.AddVert(vertex);

					vIndex = i * (4+3+3);
					vh.AddTriangle(vIndex, vIndex + 1, vIndex + 2);     // bar triangle 1
					vh.AddTriangle(vIndex + 2, vIndex + 3, vIndex);     // bar triangle 2
					vh.AddTriangle(vIndex + 4, vIndex + 5, vIndex + 6); // band peak triangle
					vh.AddTriangle(vIndex + 7, vIndex + 8, vIndex + 9); // band amp triangle
				}
			}
			else
            {
				float barWidth = _width / size;
				for (int i = 0; i < size; i++)
				{
					vertex.color = new Color(1f * samples[i], 1f - samples[i], 1f, 1.0f);
					if (i == sampleIndexTrigger1)
						vertex.color = AVD.trigger1Color;
					if (i == sampleIndexTrigger2)
						vertex.color = AVD.trigger2Color;
					vertex.position = new Vector3(i * barWidth, 0);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth, samples[i] * _height);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth, samples[i] * _height);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth, 0);
					vh.AddVert(vertex);

					// peaks
					vertex.color = Color.white;
					vertex.position = new Vector3(i * barWidth, samples_peak[i] * _height);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth * 0.5f, samples_peak[i] * _height + 8f);
					vh.AddVert(vertex);
					vertex.position = new Vector3(i * barWidth + barWidth, samples_peak[i] * _height);
					vh.AddVert(vertex);

					vIndex = i * 7;
					vh.AddTriangle(vIndex, vIndex + 1, vIndex + 2);     // bar triangle 1
					vh.AddTriangle(vIndex + 2, vIndex + 3, vIndex);     // bar triangle 2
					vh.AddTriangle(vIndex + 4, vIndex + 5, vIndex + 6); // peak triangle
				}
			}
		}
*/