﻿// ***********************************************************************************
// SallySoundFX
// ***********************************************************************************
// Created by Sally Whitemane on VAM Hub
// https://www.patreon.com/sallyw
// Licensed under CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/

// Give the class and cs-file the same name and edit it in the cslist-file
// so everything loads properly.

using System;
using UnityEngine;
using UnityEngine.UI;

namespace Sally
{
    public class AVSLAVE_name : MVRScript
	{
		// IMPORTANT - DO NOT make custom enums. The dynamic C# complier crashes Unity when it encounters these for
		// some reason

		// IMPORTANT - DO NOT OVERRIDE Awake() as it is used internally by MVRScript - instead use Init() function which
		// is called right after creation

		// UI
		private JSONStorableString jsBenchmark;

		// BENCHMARK keep namespace, can cause VAM compilation error
		private System.Diagnostics.Stopwatch stopWatch = new System.Diagnostics.Stopwatch();
		private double[] avgUpdateTimes = new double[80];
		private double avgUpdateTime = 0;

		// AUDIO VISUALIZATION DATA
		private AVD avd = new AVD();

		GameObject cube;

        public override void Init()
		{
			try
			{
				if (AVSlave.GetMasterScript(this) == null)
					SuperController.LogMessage($"{name} could not find SallySoundFX Plugin to synchronize. It will retry automatically.");

				//////////////////////////////////////////////////////////////////////////////////
				// UI - LEFT
				//////////////////////////////////////////////////////////////////////////////////

				// create custom JSON storable params here if you want them to be stored with scene JSON
				// types are JSONStorableFloat, JSONStorableBool, JSONStorableString, JSONStorableStringChooser
				// JSONStorableColor
				CreateTextField(new JSONStorableString("Text","Example Plugin Template for SallySoundFX 6.\nMakes cube bounce to bass."));

				// Benchmark
				jsBenchmark = new JSONStorableString("Benchmark", "no data");
				UIDynamicTextField tf = CreateTextField(jsBenchmark, false);
				LayoutElement le = tf.GetComponent<LayoutElement>();
				le.minHeight = 40.0f;
				le.preferredHeight = 40.0f;

				//////////////////////////////////////////////////////////////////////////////////
				// VISUALIZATION
				//////////////////////////////////////////////////////////////////////////////////

				CreateVisualization(true);
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		private void CreateVisualization(bool initialize = false)
        {
            // put code in here
            cube = GameObject.CreatePrimitive(PrimitiveType.Cube);
			// Remove next line for cube collision
			UnityEngine.Object.DestroyImmediate(cube.GetComponent<Collider>());
            MeshRenderer r = cube.GetComponent<MeshRenderer>();
            if (r == null)
            {
            	SuperController.LogError("CreateVisualization() MeshRenderer not found.");
            	return;
            }
            // Use VaM's Skybox shader "Marmoset/Skydome IBL".
			// Will change texture with skybox choice in [Scene Lighting]-tab.
            Shader shader = Shader.Find("Marmoset/Skydome IBL");
			if (shader == null)
            {
                SuperController.LogError("CreateVisualization() Shader not found.");
                return;
            }
            // Create material from shader
            r.material = new Material(shader); 
            r.material.color = Color.white;
        }

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start()
		{
			try
			{
				// put code in here
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		// Update is called with each rendered frame by Unity
		void Update()
		{
			try
			{
				// BENCHMARK START
				stopWatch.Reset();
				stopWatch.Start();

				if (!AVSlave.IsSynchronized(this))
					return;

				// put code in here
				Vector3 atomPos = containingAtom.mainController.transform.position;
                // move cube on y axis by low bass frequency sample in avd.samples[2]
                cube.transform.position = new Vector3(atomPos.x, atomPos.y + avd.samples[2], atomPos.z);

				// BENCHMARK END
				stopWatch.Stop();
				avgUpdateTimes[avgUpdateTimes.Length - 1] = stopWatch.Elapsed.TotalMilliseconds;
				Array.Copy(avgUpdateTimes, 1, avgUpdateTimes, 0, avgUpdateTimes.Length - 1); // shift array
				avgUpdateTime = 0;
				for (int i = 0; i < avgUpdateTimes.Length; i++)
					avgUpdateTime += avgUpdateTimes[i];
				avgUpdateTime = avgUpdateTime / avgUpdateTimes.Length;

				jsBenchmark.val = $"Update Benchmark: {avgUpdateTime:F4} ms";

			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate()
		{
			try
			{
				// put code in here
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy()
		{
			try
			{
                // put code in here
                Destroy(cube);
            }
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}

		//////////////////////////////////////////////////////////////////////////////////
		// UI Callbacks
		//////////////////////////////////////////////////////////////////////////////////

		// void, empty, nada, nihilum, nothing, gone, vaporized, stolen by Gremlins

		//////////////////////////////////////////////////////////////////////////////////
		// AVD Synchronization - called by SendMessage from Master
		//////////////////////////////////////////////////////////////////////////////////
		public void Sync(float[] syncdata)
		{
			try
			{
				avd.UnpackSyncdata(syncdata);
			}
			catch (Exception e) { SuperController.LogError("Exception: " + e); }
		}
	}
}
